#include "korg.hpp"
#include "korgtext.hpp"
#include <string.h>

char* arrname;
FILE* arrf;
ARRHEAD arrhead;
KORGNAME name;
int userarrange = -1;
int arrangecount = 0;

void dumparrnames()
{
KORGNAME s;

  arrangecount = (int)(arrhead.arrnames.len / KORGNAME_SIZE);

  printf("%d ARRANGEMENT NAMES: \n", arrangecount);

  fseek(arrf, arrhead.arrnames.adr, SEEK_SET);
  for (int i = 0; i < arrangecount && i < 64; i++)
  {
    if (!read_korgname(arrf, s))
      return;
    printf("  %4s  %10s", place2str(i), s);
    if (i % 4 == 3)
      printf("\n");
  }
  printf("\n");
}

const char* varchange2str(int i)
{
  switch(i)
  {
  case 0: return "OFF";
  case 1: return "->1";
  case 2: return "->2";
  case 3: return "->3";
  case 4: return "->4";
  case 5: return "1->2";
  case 6: return "1->3";
  case 7: return "1->4";
  case 8: return "2->3";
  case 9: return "2->4";
  case 10: return "3->4";
  case 11: return "UP";
  case 12: return "DOWN";
  }
  return "???";
}

const char* wrap2str(int i)
{
static char num[3];

  if (i == -1)
    return "STY";
  sprintf(num, "%02d", i+1);
  return (const char*)num;
}

void dumptrackparam(TRKPARAM& par)
{
  printf("%-5s V%03d %-6s %s OCT=%d wrap-around=%-3s  ",
    place2str(par.prog),
    par.volume,
    pan2str(par.panpot),
    efflev2str(par.effectsendlevel, 1),
    par.octave,
    wrap2str(par.wraparound));
  if (par.state & STATE_MUTE_MASK)
    printf("  MUTE");
  printf("  output=");
  switch(par.state & STATE_OUTPUT_MASK)
  {
  case 0: printf("NONE"); break;
  case 1: printf("INT"); break;
  case 2: printf("EXT"); break;
  case 3: printf("BOTH"); break;
  }
  printf("\n");
}

void dumparrangedata(ARRDATA & arr)
{
int i;

  printf("STYLE: %s\n", style2str(arr.style));
  printf("variation: %02X\n", arr.variation);
  printf("intro: %02X\n", arr.intro);
  printf("kbdassign: %02X\n", arr.kbdassign);
  printf("split: %02X\n", arr.split);
  printf("octave: %02X\n", arr.octave);
  printf("transpose: %02X\n", arr.transpose);

  printf("DRUM: %d\n", arr.drum+1);
  printf("TEMPO: %d\n", arr.tempo + 30);
  printf("SPLIT: %s\n", note2str(arr.split, -1));
  printf("Default drum mapping: ");
  for (i = 0; i < 4; i++)
    printf("Var%d=%d ", i+1, arr.drummap[i]+1);
  printf("\n");
/*
  printf("Kick Designation=%c  Snare Designation=%c\n",
    arr.kickdesignation + 'A',
    arr.snaredesignation + 'A');
*/
  printf("reserved: ");
  for (i = 0; i < 4; i++)
    printf("%02X ", arr.reserve[i]);
  printf("\n");
  printf("Variation change on Fill1: %s\n", varchange2str(arr.fill1));
  printf("Variation change on Fill2: %s\n", varchange2str(arr.fill2));

  for (i = 0; i < 6; i++)
  {
    printf("%s: ", track2str(i));
    dumptrackparam(arr.trkparam[i]);
  }

  for (i = 0; i < 2; i++)
  {
    printf("KBD%d: ", i+1);
    dumptrackparam(arr.kbdparam[i]);
    printf("  Vel. Window:  %d...%d\n",
      arr.kbdwin[i].minvel, 
      arr.kbdwin[i].maxvel);
  }
}

void dumparr()
{
ARRDATA arr;

  if (!arrheader(arrf, arrhead))
  {
    fprintf(stderr, "%s is not a KORG arr file\n", arrname);
    return;
  }
  printf("\nKORG ARRANGEMENT file %s\n\n", arrname);

  dumparrnames();

  if (sizeof(arr) != 131)
    putchar(7);
  for (userarrange = 0; userarrange < arrangecount; userarrange++)
  {
    if (!KORGREAD(arrf, arr))
      return;


    printf("ARRANGMENT  %4s  %10.10s\n", place2str(userarrange), arr.name);
    dumparrangedata(arr);
  }
}

void main(int argc, char**argv)
{
  argc--; argv++;

  if (argc == 0)
    return;

  arrname = *argv++; argc--;

  arrf = fopen(arrname, "rb");
  if (!arrf)
  {
    perror(arrname);
    return;
  }

  dumparr();

  fclose(arrf);
}
